//  URL   
// : NetSpirit

unit URI;

interface

uses
  Windows, KOL, Share;

type

  TURIOptionalFields = (ofAuthInfo, ofHost, ofParams, ofBookmark);
  TURIOptionalFieldsSet = set of TURIOptionalFields;
  
  TURI = class
    protected
      FDocument: string;
      FProtocol: string;
      FURI: string;
      FPort: string;
      Fpath: string;
      FHost: string;
      FBookmark: string;
      FUserName: string;
      FPassword: string;
      FParams: string;

      procedure SetURI(const Value: string);
      function GetURI: string;
      
    public
      constructor Create(const AURI: string = '');
      function GetFullURI(const AOptionalFileds: TURIOptionalFieldsSet = [ofAuthInfo, ofHost, ofParams, ofBookmark]): string;
      
      property Bookmark : string read FBookmark write FBookMark;
      property Document: string read FDocument write FDocument;
      property Host: string read FHost write FHost;
      property Password: string read FPassword write FPassword;
      property Path: string read FPath write FPath;
      property Params: string read FParams write FParams;
      property Port: string read FPort write FPort;
      property Protocol: string read FProtocol write FProtocol;
      property URI: string read GetURI write SetURI;
      property Username: string read FUserName write FUserName;
  end;


implementation


function RPos(const ASub, AIn: string; AStart: Integer = -1): Integer;
var
  I: Integer;
  LStartPos: Integer;
  LTokenLen: Integer;
begin
  Result := 0;
  LTokenLen := Length(ASub);
  if AStart = -1 then
  begin
    AStart := Length(AIn);
  end;
  if AStart < (Length(AIn) - LTokenLen + 1) then
  begin
    LStartPos := AStart;
  end
  else
  begin
    LStartPos := (Length(AIn) - LTokenLen + 1);
  end;
  for I := LStartPos downto 1 do
  begin
    if StrComp(PChar(Copy(AIn, I, LTokenLen)), PChar(ASub)) = 0 then
    begin
      Result := I;
      Break;
    end;
  end;
end;

procedure NormalizePath(var APath: string);
var
  I: Integer;
begin
  // Normalize the directory delimiters to follow the UNIX syntax
  I := 1;
  while I <= Length(APath) do
  begin
    if APath[I] = '\' then APath[I] := '/';
    Inc(I, 1);
  end;
end;



constructor TURI.Create(const AURI: string = '');
begin
  inherited Create;
  URI := AURI;
end;

procedure TURI.SetURI(const Value: string);
var
  I, K: Integer;
begin
  FURI := Value;
  NormalizePath(FURI);
  
  FHost := '';
  FProtocol := '';
  FPath := '';
  FDocument := '';
  FPort := '';
  FBookmark := '';
  FUsername := '';
  FPassword := '';
  FParams := '';
  
  K := 1;

  // 
  I := Pos('://', FURI);
  if I > 0 then
  begin
    FProtocol := Copy(FURI, K, I  - 1);
    FProtocol := LowerCase(FProtocol);
    K := I + 3;
  end;
  
  // 
  I := PosEx('/', FURI, K);
  if I = 0 then I := Length(FURI) + 1;
  
  if I > 0 then
  begin
    FHost := Copy(FURI, K, I - K);
    K := I;
    
    //    
    I := Pos('@', FHost);
    if I > 0 then
    begin
      FUserName := Copy(FHost, 1, I - 1);
      FHost := Copy(FHost, I + 1, MaxInt);
      
      I := Pos(':', FUserName);
      if I > 0 then
      begin
        if I = 1 then
          FUserName := '' // Ignore cases where there is only password (http://:password@host/pat/doc)
        else
        begin
          FPassword := Copy(FUserName, I + 1, MaxInt);
          SetLength(FUserName, I - 1);
        end;
      end;
    end;
    
    //  
    I := Pos(':', FHost);
    if I > 0 then
    begin
      FPort := Copy(FHost, I + 1, MaxInt);
      FHost := Copy(FHost, 1, I - 1);
    end;
    
    FHost := LowerCase(FHost);
  end;
  
  FPath := Copy(FURI, K, MaxInt);
  
  
  // 
  I := Pos('?', FPath); //  
  K := PosEx('#', FPath, I + 1); //  Bookmark / 
  if I > 0 then
  begin
    if K = 0 then
      FParams := Copy(FPath, I + 1, MaxInt)
    else
      FParams := Copy(FPath, I + 1, K - I - 1);
  end
  else
    I := K;
  
  // Bookmark
  if K > 0 then
  begin
    FBookmark := Copy(FPath, K + 1, MaxInt);
  end;
  
  //  I -      Bookmark  FPath
  if I = 0 then I := Length(FPath) + 1;
 
  //   
  K := RPos('/', FPath, I);
  FDocument := Copy(FPath, K + 1, I - K - 1);
  if K > 0 then
    SetLength(FPath, K)
  else
    FPath := '/';
end;


function TURI.GetFullURI(const AOptionalFileds: TURIOptionalFieldsSet): string;
begin
  Result := '';
  if (ofHost in AOptionalFileds) and (FHost <> '') then // If Host specified, we can return Proto, User, Pass, Host and Port
  begin
    if FProtocol <> '' then
    begin
      Result := FProtocol + '://';
    end;

    if (ofAuthInfo in AOptionalFileds) and (FUserName <> '') then
    begin
      Result := Result + FUserName;

      if FPassword <> '' then
      begin
        Result := Result + ':' + FPassword;
      end;

      Result := Result + '@';
    end;

    Result := Result + FHost;
    
    if FPort <> '' then
    begin
      Result := Result + ':' + FPort;
    end;
  end;
  
  Result := Result + FPath + FDocument;
  if (ofParams in AOptionalFileds) and (FParams <> '') then
  begin
    Result := Result + '?' + FParams;
  end;
  
  if (ofBookmark in AOptionalFileds) and (FBookmark <> '') then
  begin
    Result := Result + '#' + FBookmark;
  end;
end;

function TURI.GetURI: string;
begin
  // result must contain only the proto://host/path/document
  // If you need the full URI then you have to call GetFullURI
  Result := GetFullURI([ofHost]);
end;


end.